# -*- coding: utf-8 -*-

"""
Discord API Wrapper
~~~~~~~~~~~~~~~~~~~

A basic wrapper for the Discord API.

:copyright: (c) 2015-2017 JustMaffie
:license: MIT, see LICENSE for more details.

"""
from .core import *
from .models import *
from .http import *
from .vars import *